//+------------------------------------------------------------------+
//|                                                        Genie.mq4 |
//|                                                   Genaro Gravoso |
//|                          http://www.oocities.org/ggravoso/b.html |
//+------------------------------------------------------------------+
#property copyright "Genaro Gravoso"
#property link      "http://www.oocities.org/ggravoso/b.html"
#property description "Relative Strength Index Stochastic expert advisor"
#property version   "1.00"
#property strict

#define MAGICMA  20140801
//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
// #define MacrosHello   "Hello, world!"
// #define MacrosYear    2005

//+------------------------------------------------------------------+
//| DLL imports                                                      |
//+------------------------------------------------------------------+
// #import "user32.dll"
//   int      SendMessageA(int hWnd,int Msg,int wParam,int lParam);

// #import "my_expert.dll"
//   int      ExpertRecalculate(int wParam,int lParam);
// #import

//+------------------------------------------------------------------+
//| EX4 imports                                                      |
//+------------------------------------------------------------------+
// #import "stdlib.ex4"
//   string ErrorDescription(int error_code);
// #import
//+------------------------------------------------------------------+
//--- Inputs
input double TakeProfit    =1200;
input double Lots          =0.01;
input double MaximumRisk   =0.02;
input double DecreaseFactor=3;
input double StopLoss      =300;
extern double TrailingStart = 50; 
extern double TrailingStop = 25;
extern double TrailingStep = 5;
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//---
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//--- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//--- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//--- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
           {
            Print("Error in history!");
            break;
           }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL)
            continue;
         //---
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1)
         lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//--- return lot size
   if(lot<0.01) lot=0.01;
   return(lot);
  }

//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double RSI,S3,S50,M50;
   int    res;
//--- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//--- to simplify the coding and speed up access data are put into internal variables
   RSI=iRSI(NULL,0,4,PRICE_CLOSE,0);
   M50=iStochastic(NULL,0,50,18,3,MODE_SMA,STO_LOWHIGH,MODE_MAIN,0);
   S50=iStochastic(NULL,0,50,18,3,MODE_SMA,STO_LOWHIGH,MODE_SIGNAL,0);
   S3=iStochastic(NULL,0,5,1,1,MODE_SMA,STO_LOWHIGH,MODE_SIGNAL,0);
//--- sell conditions
   if(S50<22 && M50>S50 && S3<22 && M50>22 && S3<S50)
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,6,Bid+StopLoss*Point,Bid-TakeProfit*Point,"Genie",MAGICMA,0,Red);
      return;
     }
//--- buy conditions
   if(S50>78 && M50<S50 && S3>78 && M50<78 && S3>S50)
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,6,Ask-StopLoss*Point,Ask+TakeProfit*Point,"Genie",MAGICMA,0,Blue);
      return;
     }
//---
  }
  //+------------------------------------------------------------------+
//| Check for trailing stop order conditions                                 |
//+------------------------------------------------------------------+
void trail2(int ticket,int t1=150,int t2=80, int t3=150, int t4=80)
  {
   double sl;
   double minpts;
   minpts=StopPoint();
   if(!OrderSelect(ticket,SELECT_BY_TICKET)) return;
   if(OrderCloseTime()!=0)   return;
   
   
   //get basic info
   double oPrice,oStop,oTake,oTicket;
   oPrice=OrderOpenPrice();
   oStop=OrderStopLoss();
   oTake=OrderTakeProfit();
   oTicket=OrderTicket();   
   
   
   if(OrderType()==OP_BUY)
   {
   
      if(oStop<oPrice || oStop==0)
      {
         if(Bid-oPrice>pts(t1))
         {
            sl=oPrice+pts(t2);

         }   
         if(sl-oPrice>minpts && Bid-sl>minpts)
         {
            ModifySL(oTicket,sl);
         }
      }
      
      //trailing
      if(oStop>oPrice)
      {
         if(Bid-oStop>pts(t3))
         {
            sl=oStop+pts(t4);
 
         }   
         if(sl-oStop>minpts && Bid-sl>minpts)
         {
            ModifySL(oTicket,sl);
         }
      }
   }
   
   if(OrderType()==OP_SELL)
   {
      if(oStop>oPrice || oStop==0)
      {
         if(oPrice-Ask>pts(t1))
         {
            sl=oPrice-pts(t2);
         }   
         if(oPrice-sl>minpts && sl-Ask>minpts)
         {
            ModifySL(oTicket,sl);
         }
      }
      
      //trailing
      if(oStop<oPrice)
      {
         if(oStop-Ask>pts(t3))
         {
            sl=oStop-pts(t4);
         }   
         if(oStop-sl>minpts && sl-Ask>minpts)
         {
            ModifySL(oTicket,sl);
         }
      }
   }
//+------------------------------------------------------------------+
//| OnTick function                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false)
      return;
//--- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
//---
  }
//+------------------------------------------------------------------+